const { exec } = require('child_process');
const which = require('which');

module.exports.exec_command = function(config, host, command, command_delay, remote_upload_script, runs, timeout, callback, reconnect_callback){
  if(host.ssh){
    exec_command_ssh(config, host, command, command_delay, remote_upload_script, runs, timeout, callback, reconnect_callback)
  }else{
    exec_command_local(config, command, command_delay, remote_upload_script, runs, timeout, callback)
  }
}

function exec_command_local(config, command, command_delay, remote_upload_script, runs, timeout, callback){
  setTimeout(()=>{
    let i = 0;

    let lastError = '';
    let lastStderr = '';
    let lastStdout = '';

    let command_callback = function(){
      if(i < runs){
        let commandWithTimeout = 'timeout ' + timeout + ' ' + command;

        if(config.debug){
          console.log(commandWithTimeout)
        }

        exec(commandWithTimeout, { maxBuffer: 2000 * 1024 }, (error, stdout, stderr) => {
          if(error || stderr){
            i++;

            lastError = error;
            lastStderr = stderr;
            lastStdout = stdout;

            command_callback();
          }else{
            //NO MORE error
            callback({error: error, stdout:stdout, stderr:stderr});
          }
        });
      }else{
        //MULTIPLE TRIES FAILED
        callback({error: lastError, stdout:lastStderr, stderr:lastStdout});
      }
    }

    command_callback();
  }, command_delay*1000);
}

function exec_command_ssh(config, host, command, command_delay, remote_upload_script, runs, timeout, callback, reconnect_callback){
  setTimeout(()=>{
    let i = 0;

    let lastError = '';
    let lastStderr = '';
    let lastStdout = '';

    let command_callback = function(){
      if(i < runs){
        let commandWithTimeout = 'timeout ' + timeout + ' ' + command;

        if(config.debug){
          console.log(commandWithTimeout)
        }

        let run = function(){
          host.ssh.execCommand(commandWithTimeout).then((result) => {
            let error = '';
            let stderr = result.stderr;
            let stdout = result.stdout;

            if(error || stderr){
              i++;

              lastError = error;
              lastStderr = stderr;
              lastStdout = stdout;

              command_callback();
            }else{
              //NO MORE error
              callback({error: error, stdout:stdout, stderr:stderr});
            }
          }).catch((e) => {
            console.log(e);

            reconnect_callback(host, e);
          });
        }

        if(remote_upload_script){
          which(command, (err, result) => {
            if(err){
              console.log('Could not find script to upload');
              console.log(err);

              callback({error: lastError, stdout:lastStderr, stderr:lastStdout});
            }else{
              host.ssh.putFile(result, command).then(() => {
                commandWithTimeout = 'timeout ' + timeout + ' bash ' + command;

                run();
              }, (err) => {
                console.log('Could not upload script');
                console.log(err);

                callback({error: lastError, stdout:lastStderr, stderr:lastStdout});
              }).catch((e) => {
                console.log(e);

                reconnect_callback(host, e);
              });
            }
          });
        }else{
          run();
        }
      }else{
        //MULTIPLE TRIES FAILED
        callback({error: lastError, stdout:lastStderr, stderr:lastStdout});
      }
    }

    command_callback();
  }, command_delay*1000);
}
