const exec = require('./exec.js');
const parser = require('./parser.js');
const methods = require('./methods.js');

let run_count = 0;

module.exports = function (config, host, commands, notification_callback, callback, reconnect_callback){
  run_count++;

  let i = 0;

  let loop = function(){
    if(i < host.check_commands.length){
      let check_command = host.check_commands[i];

      if(!commands[check_command.command_name]){
        console.log('[' + host.name + ']' + ' Could not find command: ' + check_command.command_name);
        callback();
      }else{
        let command = commands[check_command.command_name];

        command = parser.parse_base64_command(command);

        command = parser.parse_required_vars_command(command, check_command);

        if(config.debug){
          console.log(command);
        }

        if(!command){
          console.log('[' + host.name + ']' + ' Not enough vars for command!');

          i++;
          loop();
        }else{
          if(!command.every || run_count % command.every === 0){
            console.log('[' + host.name + ']' + ' Running command: ' + check_command.command_name);

            let runs = config.validate_error

            if(command.retry === false){
              runs = 1
            }

            exec.exec_command(config, host, command.command, config.command_delay, command.remote_upload_script, runs, config.command_timeout, (result) => {
              let error_or_warning = {};

              if(result.error){
                error_or_warning = { state: 'error', message: 'error:\n\n' + result.error + '\nstdout:\n\n' + result.stdout};
              }else if(result.stderr){
                error_or_warning = { state: 'error', message: 'stderr:\n\n' + result.stderr + '\nstdout:\n\n' + result.stdout};
              }else{
                error_or_warning = methods.check_for_method(result.error, result.stderr, result.stdout, 'error', command.failure_on, command.failure_value);

                if(!error_or_warning){
                  error_or_warning = methods.check_for_method(result.error, result.stderr, result.stdout, 'warning', command.warning_on, command.warning_value);
                }

                if(!error_or_warning){
                    //COMMAND IS STATE OK
                    error_or_warning = { state: 'ok', message: 'stdout:\n\n' + result.stdout};
                }
              }

              let debug_command_callback = function(){
                if(command.multiple_lines_multiple_notifications){
                  //send a notification for every line of stdout
                  parser.parse_multiline_stdout(config, host, check_command, result.stdout, error_or_warning, notification_callback);
                }else{
                  notification_callback(host, check_command, error_or_warning.state, error_or_warning.message, result.stdout);
                }
              }

              if(command.debug_command){
                exec.exec_command(config, host, command.debug_command, command.remote_upload_debug_script, 0, 1, config.command_timeout,(debug_result)=>{
                  error_or_warning.message += '\n\nDebug information:\n\n' + 'stdout:\n\n' +  debug_result.stdout + '\nstderr:\n\n' + debug_result.stderr + '\n';

                  debug_command_callback();
                }, reconnect_callback);
              }else{
                debug_command_callback();
              }

              i++;
              loop();

            }, reconnect_callback);
          }else{
            console.log('[' + host.name + ']' + ' Skipping command: ' + check_command.command_name);

            i++;
            loop();
          }
        }
      }
    }else{
      callback();
    }
  }

  loop();
}
